#include <algorithm>
#include <cassert>
#include <cstdlib>
#include <chrono>
#include <iostream>
#include <ostream>
#include <string>
#include <vector>
using namespace std;

#define NOF_ELEM 100000
#define STR_LEN  256

typedef char fixed_string[STR_LEN+1];

const char * gen_str() {
    static fixed_string s;
    for (size_t i = 0; i < sizeof(fixed_string)-1; ++i) {
        s[i] = rand() % 25 + 65;
    }
    s[sizeof(s)-1] = '\0';
    return s;
}

static chrono::time_point<chrono::high_resolution_clock>
get_time ()
{
    return chrono::high_resolution_clock::now();
}

template<typename T>
chrono::microseconds
time_diff (const T& begin, const T& end)
{
    return chrono::duration_cast<chrono::microseconds>(end - begin);
}

int cmp_str(const void * x, const void * y) { return strcmp((const char *)x, (const char *)y); }

int main()
{
    fixed_string * arr1 = (fixed_string *)malloc(sizeof(fixed_string) * NOF_ELEM);
    vector<string> arr2(NOF_ELEM);

    for (size_t i = 0; i < NOF_ELEM; ++i) {
        const char * s = gen_str();
        strlcpy(arr1[i], s, sizeof(*arr1));
        arr2[i] = s;
    }
    
    /* qsort */
    auto t1 = get_time();
    qsort(arr1, NOF_ELEM, sizeof(fixed_string), cmp_str);
    cout << "qsort    : " << time_diff(t1, get_time()).count() << " microsecs" << endl;

    /* std::sort */
    t1 = get_time();
    sort(begin(arr2), end(arr2));
    cout << "std::sort: " << time_diff(t1, get_time()).count() << " microsecs" << endl;

    for (auto i = 0; i < NOF_ELEM; ++i) {
        assert(arr2[i] == arr1[i]);
    }

}
